@@ -29,11 +29,11 @@ using std::vector;
 #include <fst/cache.h>
 #include <fst/compose-filter.h>
 #include <fst/lookahead-filter.h>
-#include <fst/map.h>
 #include <fst/matcher.h>
 #include <fst/state-table.h>
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
 // Delayed composition options templated on the arc type, the matcher,
@@ -122,20 +122,22 @@ class ComposeFstImplBase : public CacheImpl<A> {
 
   ComposeFstImplBase(const Fst<A> &fst1, const Fst<A> &fst2,
                      const CacheOptions &opts)
-      :CacheImpl<A>(opts) {
+      : CacheImpl<A>(opts) {
     VLOG(2) << "ComposeFst(" << this << "): Begin";
     SetType("compose");
 
-    if (!CompatSymbols(fst2.InputSymbols(), fst1.OutputSymbols()))
-      LOG(FATAL) << "ComposeFst: output symbol table of 1st argument "
+    if (!CompatSymbols(fst2.InputSymbols(), fst1.OutputSymbols())) {
+      FSTERROR() << "ComposeFst: output symbol table of 1st argument "
                  << "does not match input symbol table of 2nd argument";
+      SetProperties(kError, kError);
+    }
 
     SetInputSymbols(fst1.InputSymbols());
     SetOutputSymbols(fst2.OutputSymbols());
   }
 
   ComposeFstImplBase(const ComposeFstImplBase<A> &impl)
-      : CacheImpl<A>(impl) {
+      : CacheImpl<A>(impl, true) {
     SetProperties(impl.Properties(), kCopyProperties);
     SetInputSymbols(impl.InputSymbols());
     SetOutputSymbols(impl.OutputSymbols());
@@ -218,40 +220,7 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
 
  public:
   ComposeFstImpl(const FST1 &fst1, const FST2 &fst2,
-                 const ComposeFstImplOptions<M1, M2, F, T> &opts)
-      : ComposeFstImplBase<Arc>(fst1, fst2, opts),
-        filter_(opts.filter ? opts.filter :
-                new F(fst1, fst2, opts.matcher1, opts.matcher2)),
-        matcher1_(filter_->GetMatcher1()),
-        matcher2_(filter_->GetMatcher2()),
-        fst1_(matcher1_->GetFst()),
-        fst2_(matcher2_->GetFst()),
-        state_table_(opts.state_table ? opts.state_table :
-                     new T(fst1_, fst2_)) {
-    MatchType type1 = matcher1_->Type(false);
-    MatchType type2 = matcher2_->Type(false);
-    if (type1 == MATCH_OUTPUT && type2  == MATCH_INPUT) {
-      match_type_ = MATCH_BOTH;
-    } else if (type1 == MATCH_OUTPUT) {
-      match_type_ = MATCH_OUTPUT;
-    } else if (type2 == MATCH_INPUT) {
-      match_type_ = MATCH_INPUT;
-    } else if (matcher1_->Type(true) == MATCH_OUTPUT) {
-      match_type_ = MATCH_OUTPUT;
-    } else if (matcher2_->Type(true) == MATCH_INPUT) {
-      match_type_ = MATCH_INPUT;
-    } else {
-      LOG(FATAL) << "ComposeFst: 1st argument cannot match on output labels "
-                 << "and 2nd argument cannot match on input labels (sort?).";
-    }
-    uint64 fprops1 = fst1.Properties(kFstProperties, false);
-    uint64 fprops2 = fst2.Properties(kFstProperties, false);
-    uint64 mprops1 = matcher1_->Properties(fprops1);
-    uint64 mprops2 = matcher2_->Properties(fprops2);
-    uint64 cprops = ComposeProperties(mprops1, mprops2);
-    SetProperties(filter_->Properties(cprops), kCopyProperties);
-    VLOG(2) << "ComposeFst(" << this << "): Initialized";
-  }
+                 const ComposeFstImplOptions<M1, M2, F, T> &opts);
 
   ComposeFstImpl(const ComposeFstImpl<M1, M2, F, T> &impl)
       : ComposeFstImplBase<Arc>(impl),
@@ -275,6 +244,22 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
     return new ComposeFstImpl<M1, M2, F, T>(*this);
   }
 
+  uint64 Properties() const { return Properties(kFstProperties); }
+
+  // Set error if found; return FST impl properties.
+  uint64 Properties(uint64 mask) const {
+    if ((mask & kError) &&
+        (fst1_.Properties(kError, false) ||
+         fst2_.Properties(kError, false) ||
+         (matcher1_->Properties(0) & kError) ||
+         (matcher2_->Properties(0) & kError) |
+         (filter_->Properties(0) & kError) ||
+         state_table_->Error())) {
+      SetProperties(kError, kError);
+    }
+    return FstImpl<Arc>::Properties(mask);
+  }
+
   // Arranges it so that the first arg to OrderedExpand is the Fst
   // that will be matched on.
   void Expand(StateId s) {
@@ -284,12 +269,19 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
     filter_->SetState(s1, s2, tuple.filter_state);
     if (match_type_ == MATCH_OUTPUT ||
         (match_type_ == MATCH_BOTH &&
-         NumArcs(fst1_, s1) > NumArcs(fst2_, s2)))
+         internal::NumArcs(fst1_, s1) > internal::NumArcs(fst2_, s2)))
       OrderedExpand(s, fst1_, s1, fst2_, s2, matcher1_, false);
     else
       OrderedExpand(s, fst2_, s2, fst1_, s1, matcher2_, true);
   }
 
+  const FST1 &GetFst1() { return fst1_; }
+  const FST2 &GetFst2() { return fst2_; }
+  M1 *GetMatcher1() { return matcher1_; }
+  M2 *GetMatcher2() { return matcher2_; }
+  F *GetFilter() { return filter_; }
+  T *GetStateTable() { return state_table_; }
+
  private:
   // This does that actual matching of labels in the composition. The
   // arguments are ordered so matching is called on state 'sa' of
@@ -341,7 +333,7 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
     StateTuple tuple(arc1.nextstate, arc2.nextstate, f);
     Arc oarc(arc1.ilabel, arc2.olabel, Times(arc1.weight, arc2.weight),
            state_table_->FindState(tuple));
-    CacheImpl<Arc>::AddArc(s, oarc);
+    CacheImpl<Arc>::PushArc(s, oarc);
   }
 
   StateId ComputeStart() {
@@ -361,12 +353,12 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
   Weight ComputeFinal(StateId s) {
     const StateTuple &tuple = state_table_->Tuple(s);
     StateId s1 = tuple.state_id1;
-    Weight final1 = Final(fst1_, s1);
+    Weight final1 = internal::Final(fst1_, s1);
     if (final1 == Weight::Zero())
       return final1;
 
     StateId s2 = tuple.state_id2;
-    Weight final2 = Final(fst2_, s2);
+    Weight final2 = internal::Final(fst2_, s2);
     if (final2 == Weight::Zero())
       return final2;
 
@@ -375,6 +367,10 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
     return Times(final1, final2);
   }
 
+  // Identifies and verifies the capabilities of the matcher to be used for
+  // composition.
+  void SetMatchType();
+
   F *filter_;
   Matcher1 *matcher1_;
   Matcher2 *matcher2_;
@@ -387,6 +383,88 @@ class ComposeFstImpl : public ComposeFstImplBase<typename M1::Arc> {
   void operator=(const ComposeFstImpl<M1, M2, F, T> &);  // disallow
 };
 
+template <class M1, class M2, class F, class T> inline
+ComposeFstImpl<M1, M2, F, T>::ComposeFstImpl(
+    const FST1 &fst1, const FST2 &fst2,
+    const ComposeFstImplOptions<M1, M2, F, T> &opts)
+    : ComposeFstImplBase<Arc>(fst1, fst2, opts),
+      filter_(opts.filter ? opts.filter :
+              new F(fst1, fst2, opts.matcher1, opts.matcher2)),
+      matcher1_(filter_->GetMatcher1()),
+      matcher2_(filter_->GetMatcher2()),
+      fst1_(matcher1_->GetFst()),
+      fst2_(matcher2_->GetFst()),
+      state_table_(opts.state_table ? opts.state_table :
+                   new T(fst1_, fst2_)) {
+  SetMatchType();
+  if (match_type_ == MATCH_NONE)
+    SetProperties(kError, kError);
+  VLOG(2) << "ComposeFst(" << this << "): Match type: "
+          << (match_type_ == MATCH_OUTPUT ? "output" :
+              (match_type_ == MATCH_INPUT ? "input" :
+               (match_type_ == MATCH_BOTH ? "both" :
+                (match_type_ == MATCH_NONE ? "none" : "unknown"))));
+
+  uint64 fprops1 = fst1.Properties(kFstProperties, false);
+  uint64 fprops2 = fst2.Properties(kFstProperties, false);
+  uint64 mprops1 = matcher1_->Properties(fprops1);
+  uint64 mprops2 = matcher2_->Properties(fprops2);
+  uint64 cprops = ComposeProperties(mprops1, mprops2);
+  SetProperties(filter_->Properties(cprops), kCopyProperties);
+  if (state_table_->Error()) SetProperties(kError, kError);
+  VLOG(2) << "ComposeFst(" << this << "): Initialized";
+}
+
+template <class M1, class M2, class F, class T>
+void ComposeFstImpl<M1, M2, F, T>::SetMatchType() {
+  MatchType type1 = matcher1_->Type(false);
+  MatchType type2 = matcher2_->Type(false);
+  uint32 flags1 = matcher1_->Flags();
+  uint32 flags2 = matcher2_->Flags();
+  if (flags1 & flags2 & kRequireMatch) {
+    FSTERROR() << "ComposeFst: only one argument can require matching.";
+    match_type_ = MATCH_NONE;
+  } else if (flags1 & kRequireMatch) {
+    if (matcher1_->Type(true) != MATCH_OUTPUT) {
+      FSTERROR() << "ComposeFst: 1st argument requires matching but cannot.";
+      match_type_ = MATCH_NONE;
+    }
+    match_type_ = MATCH_OUTPUT;
+  } else if (flags2 & kRequireMatch) {
+    if (matcher2_->Type(true) != MATCH_INPUT) {
+      FSTERROR() << "ComposeFst: 2nd argument requires matching but cannot.";
+      match_type_ = MATCH_NONE;
+    }
+    match_type_ = MATCH_INPUT;
+  } else if (flags1 & flags2 & kPreferMatch &&
+             type1 == MATCH_OUTPUT && type2  == MATCH_INPUT) {
+    match_type_ = MATCH_BOTH;
+  } else if (flags1 & kPreferMatch && type1 == MATCH_OUTPUT) {
+    match_type_ = MATCH_OUTPUT;
+  } else if  (flags2 & kPreferMatch && type2 == MATCH_INPUT) {
+    match_type_ = MATCH_INPUT;
+  } else if (type1 == MATCH_OUTPUT && type2  == MATCH_INPUT) {
+    match_type_ = MATCH_BOTH;
+  } else if (type1 == MATCH_OUTPUT) {
+    match_type_ = MATCH_OUTPUT;
+  } else if (type2 == MATCH_INPUT) {
+    match_type_ = MATCH_INPUT;
+  } else if (flags1 & kPreferMatch && matcher1_->Type(true) == MATCH_OUTPUT) {
+    match_type_ = MATCH_OUTPUT;
+  } else if  (flags2 & kPreferMatch && matcher2_->Type(true) == MATCH_INPUT) {
+    match_type_ = MATCH_INPUT;
+  } else if (matcher1_->Type(true) == MATCH_OUTPUT) {
+    match_type_ = MATCH_OUTPUT;
+  } else if (matcher2_->Type(true) == MATCH_INPUT) {
+    match_type_ = MATCH_INPUT;
+  } else {
+    FSTERROR() << "ComposeFst: 1st argument cannot match on output labels "
+               << "and 2nd argument cannot match on input labels (sort?).";
+    match_type_ = MATCH_NONE;
+  }
+}
+
+
 // Computes the composition of two transducers. This version is a
 // delayed Fst. If FST1 transduces string x to y with weight a and FST2
 // transduces y to z with weight b, then their composition transduces
@@ -484,14 +562,17 @@ class ComposeFst : public ImplToFst< ComposeFstImplBase<A> > {
   static Impl *CreateBase2(
       const typename M1::FST &fst1, const typename M2::FST &fst2,
       const ComposeFstImplOptions<M1, M2, F, T> &opts) {
+    Impl *impl = new ComposeFstImpl<M1, M2, F, T>(fst1, fst2, opts);
     if (!(Weight::Properties() & kCommutative)) {
       int64 props1 = fst1.Properties(kUnweighted, true);
       int64 props2 = fst2.Properties(kUnweighted, true);
-      if (!(props1 & kUnweighted) && !(props2 & kUnweighted))
-        LOG(FATAL) << "ComposeFst: Weight needs to be a commutative semiring: "
+      if (!(props1 & kUnweighted) && !(props2 & kUnweighted)) {
+        FSTERROR() << "ComposeFst: Weights must be a commutative semiring: "
                    << Weight::Type();
+        impl->SetProperties(kError, kError);
+      }
     }
-    return new ComposeFstImpl<M1, M2, F, T>(fst1, fst2, opts);
+    return impl;
   }
 
   // Create compose implementation specifying one matcher type.
@@ -510,16 +591,19 @@ class ComposeFst : public ImplToFst< ComposeFstImplBase<A> > {
     switch (LookAheadMatchType(fst1, fst2)) {  // Check for lookahead matchers
       default:
       case MATCH_NONE: {     // Default composition (no look-ahead)
+        VLOG(2) << "ComposeFst: Default composition (no look-ahead)";
         ComposeFstOptions<Arc> nopts(opts);
         return CreateBase1(fst1, fst2, nopts);
       }
       case MATCH_OUTPUT: {   // Lookahead on fst1
+        VLOG(2) << "ComposeFst: Lookahead on fst1";
         typedef typename DefaultLookAhead<Arc, MATCH_OUTPUT>::FstMatcher M;
         typedef typename DefaultLookAhead<Arc, MATCH_OUTPUT>::ComposeFilter F;
         ComposeFstOptions<Arc, M, F> nopts(opts);
         return CreateBase1(fst1, fst2, nopts);
       }
       case MATCH_INPUT: {    // Lookahead on fst2
+        VLOG(2) << "ComposeFst: Lookahead on fst2";
         typedef typename DefaultLookAhead<Arc, MATCH_INPUT>::FstMatcher M;
         typedef typename DefaultLookAhead<Arc, MATCH_INPUT>::ComposeFilter F;
         ComposeFstOptions<Arc, M, F> nopts(opts);
