@@ -22,11 +22,14 @@
 #define FST_LIB_STATE_TABLE_H__
 
 #include <deque>
+using std::deque;
 #include <vector>
 using std::vector;
 
+#include <fst/bi-table.h>
 #include <fst/expanded-fst.h>
 
+
 namespace fst {
 
 // STATE TABLES - these determine the bijective mapping between state
@@ -56,151 +59,55 @@ namespace fst {
 // struct StateTuple {
 //   typedef typename S StateId;
 //
-//   // Required constructor.
+//   // Required constructors.
 //   StateTuple();
+//   StateTuple(const StateTuple &);
 // };
 
 
 // An implementation using a hash map for the tuple to state ID mapping.
-// The state tuple T must have == defined and the default constructor
-// must produce a tuple that will never be seen. H is the hash function.
+// The state tuple T must have == defined. H is the hash function.
 template <class T, class H>
-class HashStateTable {
+class HashStateTable : public HashBiTable<typename T::StateId, T, H> {
  public:
   typedef T StateTuple;
   typedef typename StateTuple::StateId StateId;
+  using HashBiTable<StateId, T, H>::FindId;
+  using HashBiTable<StateId, T, H>::FindEntry;
+  using HashBiTable<StateId, T, H>::Size;
 
-  HashStateTable() {
-    StateTuple empty_tuple;
-  }
-
-  StateId FindState(const StateTuple &tuple) {
-    StateId &id_ref = tuple2id_[tuple];
-    if (id_ref == 0) {  // Tuple not found; store and assign it a new ID.
-      id2tuple_.push_back(tuple);
-      id_ref = id2tuple_.size();
-    }
-    return id_ref - 1;  // NB: id_ref = ID + 1
-  }
-
-  const StateTuple &Tuple(StateId s) const {
-    return id2tuple_[s];
-  }
-
-  StateId Size() const { return id2tuple_.size(); }
-
- private:
-  unordered_map<StateTuple, StateId, H> tuple2id_;
-  vector<StateTuple> id2tuple_;
-
-  DISALLOW_COPY_AND_ASSIGN(HashStateTable);
-};
-
-
-// An implementation using a hash set for the tuple to state ID
-// mapping.  The hash set holds 'keys' which are either the state ID
-// or kCurrentKey.  These keys can be mapped to tuples either by
-// looking up in the tuple vector or, if kCurrentKey, in current_tuple_
-// member. The hash and key equality functions map to tuples first.
-// The state tuple T must have == defined and the default constructor
-// must produce a tuple that will never be seen. H is the hash
-// function.
-template <class T, class H>
-class CompactHashStateTable {
- public:
-  friend class HashFunc;
-  friend class HashEqual;
-  typedef T StateTuple;
-  typedef typename StateTuple::StateId StateId;
-  typedef StateId Key;
-
-  CompactHashStateTable()
-      : hash_func_(*this),
-        hash_equal_(*this),
-        keys_(0, hash_func_, hash_equal_) {
-  }
+  HashStateTable() : HashBiTable<StateId, T, H>() {}
 
   // Reserves space for table_size elements.
-  explicit CompactHashStateTable(size_t table_size)
-      : hash_func_(*this),
-        hash_equal_(*this),
-        keys_(table_size, hash_func_, hash_equal_) {
-    id2tuple_.reserve(table_size);
-  }
+  explicit HashStateTable(size_t table_size)
+      : HashBiTable<StateId, T, H>(table_size) {}
 
-  StateId FindState(const StateTuple &tuple) {
-    current_tuple_ = &tuple;
-    typename KeyHashSet::const_iterator it = keys_.find(kCurrentKey);
-    if (it == keys_.end()) {
-      Key key = id2tuple_.size();
-      id2tuple_.push_back(tuple);
-      keys_.insert(key);
-      return key;
-    } else {
-      return *it;
-    }
-  }
-
-  const StateTuple &Tuple(StateId s) const {
-    return id2tuple_[s];
-  }
-
-  StateId Size() const { return id2tuple_.size(); }
-
- private:
-  static const StateId kEmptyKey;    // -1
-  static const StateId kCurrentKey;  // -2
-
-  class HashFunc {
-   public:
-    HashFunc(const CompactHashStateTable &ht) : ht_(&ht) {}
-
-    size_t operator()(Key k) const { return hf(ht_->Key2Tuple(k)); }
-   private:
-    const CompactHashStateTable *ht_;
-    H hf;
-  };
-
-  class HashEqual {
-   public:
-    HashEqual(const CompactHashStateTable &ht) : ht_(&ht) {}
-
-    bool operator()(Key k1, Key k2) const {
-      return ht_->Key2Tuple(k1) == ht_->Key2Tuple(k2);
-    }
-   private:
-    const CompactHashStateTable *ht_;
-  };
-
-  typedef unordered_set<Key, HashFunc, HashEqual> KeyHashSet;
-
-  const StateTuple &Key2Tuple(Key k) const {
-    if (k == kEmptyKey)
-      return empty_tuple_;
-    else if (k == kCurrentKey)
-      return *current_tuple_;
-    else
-      return id2tuple_[k];
-  }
-
-  HashFunc hash_func_;
-  HashEqual hash_equal_;
-  KeyHashSet keys_;
-  vector<StateTuple> id2tuple_;
-  const StateTuple empty_tuple_;
-  const StateTuple *current_tuple_;
-
-  DISALLOW_COPY_AND_ASSIGN(CompactHashStateTable);
+  StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
+  const StateTuple &Tuple(StateId s) const { return FindEntry(s); }
 };
 
-template <class T, class H>
-const typename CompactHashStateTable<T, H>::StateId
-CompactHashStateTable<T, H>::kEmptyKey = -1;
 
+// An implementation using a hash map for the tuple to state ID mapping.
+// The state tuple T must have == defined. H is the hash function.
 template <class T, class H>
-const typename CompactHashStateTable<T, H>::StateId
-CompactHashStateTable<T, H>::kCurrentKey = -2;
+class CompactHashStateTable
+    : public CompactHashBiTable<typename T::StateId, T, H> {
+ public:
+  typedef T StateTuple;
+  typedef typename StateTuple::StateId StateId;
+  using CompactHashBiTable<StateId, T, H>::FindId;
+  using CompactHashBiTable<StateId, T, H>::FindEntry;
+  using CompactHashBiTable<StateId, T, H>::Size;
 
+  CompactHashStateTable() : CompactHashBiTable<StateId, T, H>() {}
+
+  // Reserves space for 'table_size' elements.
+  explicit CompactHashStateTable(size_t table_size)
+      : CompactHashBiTable<StateId, T, H>(table_size) {}
+
+  StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
+  const StateTuple &Tuple(StateId s) const { return FindEntry(s); }
+};
 
 // An implementation using a vector for the tuple to state mapping.
 // It is passed a function object FP that should fingerprint tuples
@@ -209,91 +116,22 @@ CompactHashStateTable<T, H>::kCurrentKey = -2;
 // pass in this object; in that case, VectorStateTable takes its
 // ownership.
 template <class T, class FP>
-class VectorStateTable {
+class VectorStateTable
+    : public VectorBiTable<typename T::StateId, T, FP> {
  public:
   typedef T StateTuple;
   typedef typename StateTuple::StateId StateId;
-  typedef typename StateTuple::FilterState FilterState;
+  using VectorBiTable<StateId, T, FP>::FindId;
+  using VectorBiTable<StateId, T, FP>::FindEntry;
+  using VectorBiTable<StateId, T, FP>::Size;
+  using VectorBiTable<StateId, T, FP>::Fingerprint;
 
-  explicit VectorStateTable(FP *fp = 0)
-      : fp_(fp ? fp : new FP()) {}
+  // Reserves space for 'table_size' elements.
+  explicit VectorStateTable(FP *fp = 0, size_t table_size = 0)
+      : VectorBiTable<StateId, T, FP>(fp, table_size) {}
 
-  ~VectorStateTable() { delete fp_; }
-
-  StateId FindState(const StateTuple &tuple) {
-    ssize_t fp = (*fp_)(tuple);
-    if (fp >= fp2id_.size())
-      fp2id_.resize(fp + 1);
-    StateId &id_ref = fp2id_[fp];
-    if (id_ref == 0) {  // Tuple not found; store and assign it a new ID.
-      id2tuple_.push_back(tuple);
-      id_ref = id2tuple_.size();
-    }
-    return id_ref - 1;  // NB: id_ref = ID + 1
-  }
-
-  const StateTuple &Tuple(StateId s) const {
-    return id2tuple_[s];
-  }
-
-  StateId Size() const { return id2tuple_.size(); }
-
-  const FP &Fingerprint() const { return *fp_; }
-
- private:
-  FP *fp_;
-  vector<StateId> fp2id_;
-  vector<StateTuple> id2tuple_;
-
-  DISALLOW_COPY_AND_ASSIGN(VectorStateTable);
-};
-
-// An implementation using a hash map for the tuple to state ID
-// mapping. This version permits erasing of states.  The state tuple T
-// must have == defined and its default constructor must produce a
-// tuple that will never be seen. F is the hash function.
-template <class T, class F>
-class ErasableStateTable {
- public:
-  typedef T StateTuple;
-  typedef typename StateTuple::StateId StateId;
-
-  ErasableStateTable() : first_(0) {}
-
-  StateId FindState(const StateTuple &tuple) {
-    StateId &id_ref = tuple2id_[tuple];
-    if (id_ref == 0) {  // Tuple not found; store and assign it a new ID.
-      id2tuple_.push_back(tuple);
-      id_ref = id2tuple_.size() + first_;
-    }
-    return id_ref - 1;  // NB: id_ref = ID + 1
-  }
-
-  const StateTuple &Tuple(StateId s) const {
-    return id2tuple_[s - first_];
-  }
-
-  StateId Size() const { return id2tuple_.size(); }
-
-  void Erase(StateId s) {
-    StateTuple &tuple = id2tuple_[s - first_];
-    typename unordered_map<StateTuple, StateId, F>::iterator it =
-        tuple2id_.find(tuple);
-    tuple2id_.erase(it);
-    id2tuple_[s - first_] = empty_tuple_;
-    while (!id2tuple_.empty() && id2tuple_.front() == empty_tuple_) {
-      id2tuple_.pop_front();
-      ++first_;
-    }
-  }
-
- private:
-  unordered_map<StateTuple, StateId, F> tuple2id_;
-  deque<StateTuple> id2tuple_;
-  const StateTuple empty_tuple_;
-  StateId first_;        // StateId of first element in the deque;
-
-  DISALLOW_COPY_AND_ASSIGN(ErasableStateTable);
+  StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
+  const StateTuple &Tuple(StateId s) const { return FindEntry(s); }
 };
 
 
@@ -304,133 +142,47 @@ class ErasableStateTable {
 // suitable for indexing in a vector).  The hash functor H is used when
 // hashing tuple into the compact hash table.
 template <class T, class S, class FP, class H>
-class VectorHashStateTable {
+class VectorHashStateTable
+    : public VectorHashBiTable<typename T::StateId, T, S, FP, H> {
  public:
-  friend class HashFunc;
-  friend class HashEqual;
   typedef T StateTuple;
   typedef typename StateTuple::StateId StateId;
-  typedef StateId Key;
+  using VectorHashBiTable<StateId, T, S, FP, H>::FindId;
+  using VectorHashBiTable<StateId, T, S, FP, H>::FindEntry;
+  using VectorHashBiTable<StateId, T, S, FP, H>::Size;
+  using VectorHashBiTable<StateId, T, S, FP, H>::Selector;
+  using VectorHashBiTable<StateId, T, S, FP, H>::Fingerprint;
+  using VectorHashBiTable<StateId, T, S, FP, H>::Hash;
 
   VectorHashStateTable(S *s, FP *fp, H *h,
                        size_t vector_size = 0,
                        size_t tuple_size = 0)
-      : selector_(s),
-        fp_(fp),
-        h_(h),
-        hash_func_(*this),
-        hash_equal_(*this),
-        keys_(0, hash_func_, hash_equal_) {
-    if (vector_size)
-      fp2id_.reserve(vector_size);
-    if (tuple_size)
-      id2tuple_.reserve(tuple_size);
-  }
+      : VectorHashBiTable<StateId, T, S, FP, H>(
+          s, fp, h, vector_size, tuple_size) {}
 
-  ~VectorHashStateTable() {
-    delete selector_;
-    delete fp_;
-    delete h_;
-  }
-
-  StateId FindState(const StateTuple &tuple) {
-    if ((*selector_)(tuple)) {  // Use the vector if 'selector_(tuple) == true'
-      uint64 fp = (*fp_)(tuple);
-      if (fp2id_.size() <= fp)
-        fp2id_.resize(fp + 1, 0);
-      if (fp2id_[fp] == 0) {
-        id2tuple_.push_back(tuple);
-        fp2id_[fp] = id2tuple_.size();
-      }
-      return fp2id_[fp] - 1;  // NB: assoc_value = ID + 1
-    } else {  // Use the hash table otherwise.
-      current_tuple_ = &tuple;
-      typename KeyHashSet::const_iterator it = keys_.find(kCurrentKey);
-      if (it == keys_.end()) {
-        Key key = id2tuple_.size();
-        id2tuple_.push_back(tuple);
-        keys_.insert(key);
-        return key;
-      } else {
-        return *it;
-      }
-    }
-  }
-
-  const StateTuple &Tuple(StateId s) const {
-    return id2tuple_[s];
-  }
-
-  StateId Size() const { return id2tuple_.size(); }
-
-  const S &Selector() const { return *selector_; }
-
-  const FP &Fingerprint() const { return *fp_; }
-
-  const H &Hash() const { return *h_; }
-
- private:
-  static const StateId kEmptyKey;
-  static const StateId kCurrentKey;
-
-  class HashFunc {
-   public:
-    HashFunc(const VectorHashStateTable &ht) : ht_(&ht) {}
-
-    size_t operator()(Key k) const { return (*(ht_->h_))(ht_->Key2Tuple(k)); }
-   private:
-    const VectorHashStateTable *ht_;
-  };
-
-  class HashEqual {
-   public:
-    HashEqual(const VectorHashStateTable &ht) : ht_(&ht) {}
-
-    bool operator()(Key k1, Key k2) const {
-      return ht_->Key2Tuple(k1) == ht_->Key2Tuple(k2);
-    }
-   private:
-    const VectorHashStateTable *ht_;
-  };
-
-  typedef unordered_set<Key, HashFunc, HashEqual> KeyHashSet;
-
-  const StateTuple &Key2Tuple(Key k) const {
-    if (k == kEmptyKey)
-      return empty_tuple_;
-    else if (k == kCurrentKey)
-      return *current_tuple_;
-    else
-      return id2tuple_[k];
-  }
-
-
-  S *selector_;  // Returns true if tuple hashed into vector
-  FP *fp_;       // Fingerprint used when hashing tuple into vector
-  H *h_;         // Hash function used when hashing tuple into hash_set
-
-  vector<StateTuple> id2tuple_;  // Maps state IDs to tuple
-  vector<StateId> fp2id_;        // Maps tuple fingerprints to IDs
-
-  // Compact implementation of the hash table mapping tuples to
-  // state IDs using the hash function 'h_'
-  HashFunc hash_func_;
-  HashEqual hash_equal_;
-  KeyHashSet keys_;
-  const StateTuple empty_tuple_;
-  const StateTuple *current_tuple_;
-
-  DISALLOW_COPY_AND_ASSIGN(VectorHashStateTable);
+  StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
+  const StateTuple &Tuple(StateId s) const { return FindEntry(s); }
 };
 
-template <class T, class S, class FP, class H>
-const typename VectorHashStateTable<T, S, FP, H>::StateId
-VectorHashStateTable<T, S, FP, H>::kEmptyKey = -1;
 
-template <class T, class S, class FP, class H>
-const typename VectorHashStateTable<T, S, FP, H>::StateId
-VectorHashStateTable<T, S, FP, H>::kCurrentKey = -2;
+// An implementation using a hash map for the tuple to state ID
+// mapping. This version permits erasing of states.  The state tuple T
+// must have == defined and its default constructor must produce a
+// tuple that will never be seen. F is the hash function.
+template <class T, class F>
+class ErasableStateTable : public ErasableBiTable<typename T::StateId, T, F> {
+ public:
+  typedef T StateTuple;
+  typedef typename StateTuple::StateId StateId;
+  using ErasableBiTable<StateId, T, F>::FindId;
+  using ErasableBiTable<StateId, T, F>::FindEntry;
+  using ErasableBiTable<StateId, T, F>::Size;
+  using ErasableBiTable<StateId, T, F>::Erase;
 
+  ErasableStateTable() : ErasableBiTable<StateId, T, F>() {}
+  StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
+  const StateTuple &Tuple(StateId s) const { return FindEntry(s); }
+};
 
 //
 // COMPOSITION STATE TUPLES AND TABLES
@@ -454,6 +206,8 @@ VectorHashStateTable<T, S, FP, H>::kCurrentKey = -2;
 //   const StateTuple<StateId> &Tuple(StateId) const;
 //   // # of stored tuples.
 //   StateId Size() const;
+//   // Return true if error encountered
+//   bool Error() const;
 // };
 
 // Represents the composition state.
@@ -521,7 +275,11 @@ class GenericComposeStateTable : public H {
 
   GenericComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2) {}
 
-  GenericComposeStateTable(const GenericComposeStateTable<A, F> &table) {}
+  // Reserves space for 'table_size' elements.
+  GenericComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2,
+                           size_t table_size) : H(table_size) {}
+
+  bool Error() const { return false; }
 
  private:
   void operator=(const GenericComposeStateTable<A, F> &table);  // disallow
@@ -536,9 +294,9 @@ class ComposeFingerprint {
   typedef F FilterState;
   typedef ComposeStateTuple<S, F> StateTuple;
 
-  // Required, but here useless, constructor.
-  ComposeFingerprint() {
-    LOG(FATAL) << "TupleFingerprint: # of FST state must be provided.";
+  // Required but suboptimal constructor.
+  ComposeFingerprint() : mult1_(8192), mult2_(8192) {
+    LOG(WARNING) << "TupleFingerprint: # of FST states should be provided.";
   }
 
   // Constructor is provided the sizes of the input FSTs
@@ -593,17 +351,20 @@ VectorStateTable<ComposeStateTuple<typename A::StateId, F>,
   typedef typename A::StateId StateId;
   typedef F FilterState;
   typedef ComposeStateTuple<StateId, F> StateTuple;
+  typedef VectorStateTable<StateTuple,
+                           ComposeFingerprint<StateId, F> > StateTable;
 
-  ProductComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2)
-      : VectorStateTable<ComposeStateTuple<StateId, F>,
-                         ComposeFingerprint<StateId, F> >
-  (new ComposeFingerprint<StateId, F>(CountStates(fst1),
-                                      CountStates(fst2))) { }
+  // Reserves space for 'table_size' elements.
+  ProductComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2,
+                           size_t table_size = 0)
+      : StateTable(new ComposeFingerprint<StateId, F>(CountStates(fst1),
+                                                      CountStates(fst2)),
+                   table_size) {}
 
   ProductComposeStateTable(const ProductComposeStateTable<A, F> &table)
-      : VectorStateTable<ComposeStateTuple<StateId, F>,
-                         ComposeFingerprint<StateId, F> >
-        (new ComposeFingerprint<StateId, F>(table.Fingerprint())) {}
+      : StateTable(new ComposeFingerprint<StateId, F>(table.Fingerprint())) {}
+
+  bool Error() const { return false; }
 
  private:
   void operator=(const ProductComposeStateTable<A, F> &table);  // disallow
@@ -624,19 +385,29 @@ VectorStateTable<ComposeStateTuple<typename A::StateId, F>,
   typedef typename A::StateId StateId;
   typedef F FilterState;
   typedef ComposeStateTuple<StateId, F> StateTuple;
+  typedef VectorStateTable<StateTuple,
+                           ComposeState1Fingerprint<StateId, F> > StateTable;
 
-  StringDetComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2) {
+  StringDetComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2)
+      : error_(false) {
     uint64 props1 = kString;
     uint64 props2 = kIDeterministic | kNoIEpsilons;
     if (fst1.Properties(props1, true) != props1 ||
-        fst2.Properties(props2, true) != props2)
-      LOG(FATAL) << "StringDetComposeStateTable: fst1 not a string or"
+        fst2.Properties(props2, true) != props2) {
+      FSTERROR() << "StringDetComposeStateTable: fst1 not a string or"
                  << " fst2 not input deterministic and epsilon-free";
+      error_ = true;
+    }
   }
 
-  StringDetComposeStateTable(const StringDetComposeStateTable<A, F> &table) {}
+  StringDetComposeStateTable(const StringDetComposeStateTable<A, F> &table)
+      : StateTable(table), error_(table.error_) {}
+
+  bool Error() const { return error_; }
 
  private:
+  bool error_;
+
   void operator=(const StringDetComposeStateTable<A, F> &table);  // disallow
 };
 
@@ -650,25 +421,35 @@ VectorStateTable<ComposeStateTuple<typename A::StateId, F>,
 template <typename A, typename F>
 class DetStringComposeStateTable : public
 VectorStateTable<ComposeStateTuple<typename A::StateId, F>,
-                 ComposeState1Fingerprint<typename A::StateId, F> > {
+                 ComposeState2Fingerprint<typename A::StateId, F> > {
  public:
   typedef A Arc;
   typedef typename A::StateId StateId;
   typedef F FilterState;
   typedef ComposeStateTuple<StateId, F> StateTuple;
+  typedef VectorStateTable<StateTuple,
+                           ComposeState2Fingerprint<StateId, F> > StateTable;
 
-  DetStringComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2) {
+  DetStringComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2)
+      :error_(false) {
     uint64 props1 = kODeterministic | kNoOEpsilons;
     uint64 props2 = kString;
     if (fst1.Properties(props1, true) != props1 ||
-        fst2.Properties(props2, true) != props2)
-      LOG(FATAL) << "StringDetComposeStateTable: fst2 not a string or"
+        fst2.Properties(props2, true) != props2) {
+      FSTERROR() << "StringDetComposeStateTable: fst2 not a string or"
                  << " fst1 not output deterministic and epsilon-free";
+      error_ = true;
+    }
   }
 
-  DetStringComposeStateTable(const DetStringComposeStateTable<A, F> &table) {}
+  DetStringComposeStateTable(const DetStringComposeStateTable<A, F> &table)
+      : StateTable(table), error_(table.error_) {}
+
+  bool Error() const { return error_; }
 
  private:
+  bool error_;
+
   void operator=(const DetStringComposeStateTable<A, F> &table);  // disallow
 };
 
@@ -689,7 +470,7 @@ ErasableStateTable<ComposeStateTuple<typename A::StateId, F>,
 
   ErasableComposeStateTable(const Fst<A> &fst1, const Fst<A> &fst2) {}
 
-  ErasableComposeStateTable(const ErasableComposeStateTable<A, F> &table) {}
+  bool Error() const { return false; }
 
  private:
   void operator=(const ErasableComposeStateTable<A, F> &table);  // disallow
